"""
Module: adjusttable.py

2005.07 - I18n version

Copyright (C) 2005 by Matsumoto,Tadashi
(E-Mail Address: ma2@city.plala.jp)

Everyone is permitted to do anything on this program
including copying, modifying, improving,
as long as you don't try to pretend that you wrote it.
i.e., the above copyright notice has to appear in all copies.
Binary distribution requires original version messages.
You don't have to ask before copying, redistribution or publishing.

THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE.
"""

import sys, os, re

is8bit = re.compile(ur'^[\u0000-\u00ff]*$').match # less than 256

if sys.version_info[:2] >= (2, 4): # Python 2.4+
    from unicodedata import east_asian_width
    def unichr_width(u):
        w = east_asian_width(u)
        if w in ('W', 'F'): return 2
        else: return 1
else: # Python 2.3
    import mmap
    path = os.path.dirname(__file__)
    data = os.path.join(path, 'east_asian_width.data')
    N = 0x2000  # 0x10000 = N * 8
    f = open(data, 'rb')
    m = mmap.mmap(f.fileno(), N, access=mmap.ACCESS_READ)
    def unichr_width(u):
        q, r = divmod(ord(u), 8)
        if ord(m[q])&(1<<r): return 2
        else: return 1

def unistr_width(u):
    return sum([unichr_width(x) for x in u])

def _adjust_column(line, width):
    "a part of adjustsimpletable"
    w, i = 0, 0
    for c in line:
        w0 = unichr_width(c)
        if w + w0 > width:
            col, rline = line[:i], line[i:]
            diff = w - i
            if diff > 0: 
                col += ' '*diff
            return col, rline
        w += w0 
        i += 1
    return line, u''

def adjustgridtable(lines):
    rlines = []
    for line in lines:
        if is8bit(line):
            rlines.append(line)
            continue
        rline = []
        for col in line.split('|'):
            diff = unistr_width(col) - len(col)
            if diff > 0:
                col += ' '*diff
            rline.append(col)
        rlines.append('|'.join(rline))
    return rlines

def adjustsimpletable(lines, columns):
    if len(columns) <= 1:
        return lines
    rlines = []
    for line in lines:
        if is8bit(line):
            rlines.append(line)
            continue
        rline = []
        end0 = 0
        for start, end in columns[:-1]:
            width = end - end0
            col, line = _adjust_column(line, width)
            rline.append(col)
            end0 = end
        rline.append(line) # last column need not to adjust
        rlines.append(''.join(rline))
    return rlines
